<?php

namespace XFI\Import\Importer;

use XF\Import\Data\EntityEmulator;
use XF\Import\Importer\AbstractForumImporter;
use XF\Import\StepState;
use XF\Util\Arr;

class BestAnswers extends AbstractForumImporter
{
    /**
	 * @var \XF\Db\Mysqli\Adapter
	 */
	protected $sourceDb;

	/**
	 * @var array
	 */
	protected $userFields;

	const MAX_POSTS = 500;

	public static function getListInfo()
	{
		return [
			'target' => 'XenForo',
			'source' => 'BestAnswer 1.0.5',
			'beta' => true
		];
	}

	protected function getBaseConfigDefault()
	{
		return [
			'db'         => [
				'host'     => '',
				'username' => '',
				'password' => '',
				'dbname'   => '',
				'port'     => 3306,
				'tablePrefix'   => '',
				'charset'  => '', // used for the DB connection
			],
			'attachpath'     => null,
			'avatarpath'     => null,
			'blogattachpath' => null,
			'bburl'          => null,
			'charset'        => '', // used for UTF8 conversion
			'super_admins'   => [],
		];
    }
    
    public function renderBaseConfigOptions(array $vars)
	{
		if (empty($vars['fullConfig']['db']['host']))
		{
			$configPath = getcwd() . '/includes/config.php';
			if (file_exists($configPath) && is_readable($configPath))
			{
				$config = [];
				include($configPath);

				$vars['db'] = [
					'host'        => $config['MasterServer']['servername'],
					'port'        => $config['MasterServer']['port'],
					'username'    => $config['MasterServer']['username'],
					'password'    => $config['MasterServer']['password'],
					'dbname'      => $config['Database']['dbname'],
					'tablePrefix'      => $config['Database']['tableprefix'],
					'charset'     => $config['Mysqli']['charset']
				];
				$vars['super_admins'] = $config['SpecialUsers']['superadministrators'];
			}
			else
			{
				$vars['db'] = [
					'host' => $this->app->config['db']['host'],
					'port' => $this->app->config['db']['port'],
					'username' => $this->app->config['db']['username']
				];
			}
		}

		return $this->app->templater()->renderTemplate('admin:xfi_import_config_vbulletin', $vars);
	}

    public function validateBaseConfig(array &$baseConfig, array &$errors)
	{
		static $guessedPrefixes = null;

		$baseConfig['db']['tablePrefix'] = preg_replace('/[^a-z0-9_]/i', '', $baseConfig['db']['tablePrefix']);

		$fullConfig = array_replace_recursive($this->getBaseConfigDefault(), $baseConfig);
		$missingFields = false;

		if ($fullConfig['db']['host'])
		{
			$validDbConnection = false;

			try
			{
				$sourceDb = new \XF\Db\Mysqli\Adapter($fullConfig['db'], false);
				$sourceDb->getConnection();
				$validDbConnection = true;
			}
			catch (\XF\Db\Exception $e)
			{
				$errors[] = \XF::phrase('source_database_connection_details_not_correct_x', ['message' => $e->getMessage()]);
			}

			if ($validDbConnection)
			{
				try
				{
					$options = $sourceDb->fetchPairs("
						SELECT varname, value
						FROM setting
						WHERE varname IN('attachpath', 'attachfile', 'avatarpath', 'usefileavatar', 'languageid', 'bburl', 'blogattachfile', 'blogattachpath')
					");
				}
				catch (\XF\Db\Exception $e)
				{
					if ($fullConfig['db']['dbname'] === '')
					{
						$errors[] = \XF::phrase('please_enter_database_name');
					}
					else
					{
						if ($guessedPrefixes === null && $guessedPrefixes = $this->guessTablePrefix($sourceDb, $fullConfig['db']['dbname']))
						{
							if (count($guessedPrefixes) == 1)
							{
								$baseConfig['db']['tablePrefix'] = $guessedPrefixes[0];
								return $this->validateBaseConfig($baseConfig, $errors);
							}
							else
							{
								$errors['foundPrefixes'] = \XF::phrase('table_prefix_incorrect_try_x', [
									'prefixlist' => implode('   ', $guessedPrefixes)]);
							}
						}
						else
						{
							$errors[] = \XF::phrase('table_prefix_or_database_name_is_not_correct');
						}
					}
				}

				if (!empty($options))
				{
					if (!empty($options['usefileavatar']) && !empty($options['avatarpath']))
					{
						$baseConfig['avatarpath'] = $this->getBaseConfigAvatarPath($options['avatarpath']);
					}

					if (!empty($options['attachfile']) && !empty($options['attachpath']))
					{
						$baseConfig['attachpath'] = trim($options['attachpath']);
					}

					if (!empty($options['blogattachfile']) && !empty($options['blogattachpath']))
					{
						$baseConfig['blogattachpath'] = trim($options['blogattachpath']);
					}

					if (!empty($options['bburl']))
					{
						$baseConfig['bburl'] = $options['bburl'];
					}
				}

				if (!empty($options['languageid']))
				{
					$defaultCharset = $sourceDb->fetchOne("
						SELECT charset
						FROM language
						WHERE languageid = ?
					", $options['languageid']);

					if (!$defaultCharset || str_replace('-', '', strtolower($defaultCharset)) == 'iso88591')
					{
						$baseConfig['charset'] = 'windows-1252';
					}
					else
					{
						$baseConfig['charset'] = strtolower($defaultCharset);
					}
				}
			}
			else
			{
				$missingFields = true;
			}

			$baseConfig['super_admins'] = trim($fullConfig['super_admins']);

		}

		if ($missingFields)
		{
			$errors[] = \XF::phrase('please_complete_required_fields');
		}

		return $errors ? false : true;
    }
    
    protected function getBaseConfigAvatarPath($pathFromDb)
	{
		return trim($pathFromDb);
    }

    protected function getStepConfigOptions(array $vars)
	{
		$vars['stepConfig'] = $this->getStepConfigDefault();

		if (in_array('forums', $vars['steps']) || in_array('faq', $vars['steps']))
		{
			/** @var \XF\Repository\Node $nodeRepo */
			$nodeRepo = $this->app->repository('XF:Node');
			$vars['nodeTree'] = $nodeRepo->createNodeTree($nodeRepo->getFullNodeList());
		}

		if (in_array('notices', $vars['steps']))
		{
			/** @var \XF\Repository\Notice $noticeRepo */
			$noticeRepo = $this->app->repository('XF:Notice');
			$vars['noticeTypes'] = $noticeRepo->getNoticeTypes();
		}

		return $vars;
	}
    protected function getStepConfigDefault()
	{
		return [
			
		];
	}
	public function renderStepConfigOptions(array $vars)
	{
		return $this->app->templater()->renderTemplate('admin:xfi_import_step_config_vbulletin', $this->getStepConfigOptions($vars));
	}
    
    public function validateStepConfig(array $steps, array &$stepConfig, array &$errors)
	{
		foreach (array_keys($stepConfig) AS $step)
		{
			$callback = 'validateStepConfig' . ucfirst($step);

			if (method_exists($this, $callback))
			{
				$this->$callback($stepConfig[$step], $stepConfig, $errors);
			}
		}

		return $errors ? false : true;
    }
    

    /**
	 * Process super admin IDs
	 *
	 * @param array $config
	 * @param array $stepConfig
	 * @param array $errors
	 */
	protected function validateStepConfigUsers(array &$config, array &$stepConfig, array &$errors)
	{
		if (isset($config['super_admins']))
		{
			$config['super_admins'] = Arr::stringToArray($config['super_admins'], '/\s*,\s*/s');
		}
		else
		{
			$config['super_admins'] = [];
		}
    }
    
    protected function doInitializeSource()
	{
		$dbConfig = $this->baseConfig['db'];

		$this->sourceDb = new \XF\Db\Mysqli\Adapter($dbConfig, false);

		$this->dataManager->setSourceCharset($this->baseConfig['charset'], true);
	}

    public function getSteps()
	{
		return [
			'forumAnswers' => [
				'title' => \XF::phrase('bai_import_forum_answers')
            ],
            'threadAnswers' => [
				'title' => \XF::phrase('bai_import_thread_answers')
            ],

        ];
    }

    public function stepForumAnswers(StepState $state, array $stepConfig)
    {
        $forums = $this->sourceDb->fetchAll(" SELECT * FROM forum WHERE g_answer_forum <> '0' AND parentid <> '-1'");
        $ba_forum = array();
        foreach ($forums as $forum) {
            $ba_forum[] = '"'.$forum['forumid'].'"';
            $state->imported++;
        }
        $baForum = implode(",", $ba_forum);
        $updateOption = \XF::db()->query("UPDATE xf_option SET option_value = '[$baForum]' WHERE option_id = ?", 'ba_forums');
        return $state->complete();
    }

    public function stepThreadAnswers(StepState $state, array $stepConfig, $maxTime, $limit = 1000){

        $timer = new \XF\Timer($maxTime);

        $threads = $this->sourceDb->fetchAll(" SELECT thread.threadid, thread.title , thread.g_answer_forum_answer, 
        post.postid, post.username, post.userid 
        FROM thread AS thread
        STRAIGHT_JOIN post AS post 
        ON (thread.g_answer_forum_answer = post.postid)
        WHERE thread.g_answer_forum_answer <> ? 
        ORDER BY thread.threadid
        limit {$limit}
        ", 0);

        // if (!$threads)
		// {
		// 	return $threads->complete();
        // }
        
        $total = 0;
        foreach ($threads as $thread ) {
            $oldId = $thread['postid'];
            $postid = $thread['postid'];
            $userid = $thread['userid'];
            $threadid = $thread['threadid'];
            $prefix = \XF::options()->ba_prefix['action'];
            $import = \XF::db()->query("UPDATE xf_thread SET xena_post_ba = '$postid', xena_userid_ba = '$userid' , prefix_id = '$prefix'
             WHERE thread_id = ?", $threadid);
            if ($import)
			{
                $state->imported++;
                $total++;
			}

			if ($timer->limitExceeded())
			{
				break;
			}
        }

       return $state->complete();
    }
}

